//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  modulesitem.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "modulesitem.hpp"
#include "versioning.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals/Constants
//
#define IDM_COPYMODULEPATH      1001
#define IDM_COPYIDSTRING        1002
#define IDM_UNLOADMODULE        1003
#define IDM_RELOADMODULE        1004


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
ModulesItem::ModulesItem(HWND hBox, HWND hListView)
: IMenuItem(hBox, hListView)
{
}


char* FindFileName(const char* pszPath)
{
    char* pszReturn = strrchr(pszPath, '\\');

    if (pszReturn)
    {
        ++pszReturn;
    }

    return pszReturn;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ModulesCallback
//
BOOL __stdcall ModulesCallback(LPCSTR pszPath, DWORD dwFlags, LPARAM lParam)
{
    CallbackData* pCd = (CallbackData*)lParam;
    
    LVITEM itemInfo;
    itemInfo.mask = LVIF_TEXT;
    itemInfo.iItem = pCd->nItem;
    itemInfo.pszText = (char*)pszPath;
    itemInfo.iSubItem = 0;
    
    ListView_InsertItem(pCd->hListView, &itemInfo);
    
    WORD wVersion[3] = { 0 };
    
    if (GetResourceVersion(pszPath, wVersion, 3))
    {
        char szBuffer[MAX_PATH] = { 0 };

        StringCchPrintf(szBuffer, MAX_PATH, "%u.%u.%u",
            wVersion[0], wVersion[1], wVersion[2]);

        ListView_SetItemText(pCd->hListView, pCd->nItem, 1, szBuffer);
    }

    ListView_SetItemText(pCd->hListView, pCd->nItem, 2,
        (dwFlags & LS_MODULE_THREADED) ? "yes" : "no";);

    ++pCd->nItem;

    return TRUE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Attach
//
void ModulesItem::Attach()
{
    IMenuItem::Attach();
    
    LVCOLUMN columnInfo = GetBasicColumn(m_hListView, "Module");
    
    int width = GetMaxWidth(m_hListView);
    columnInfo.cx = width * 4 / 6;
    
    ListView_InsertColumn(m_hListView, 0, &columnInfo);
    
    columnInfo.cx = width * 1 / 6;
    columnInfo.pszText = "Version";
    columnInfo.iSubItem = 1;
    
    ListView_InsertColumn(m_hListView, 1, &columnInfo);

    columnInfo.cx = width * 1 / 6;
    columnInfo.pszText = "Threaded";
    columnInfo.iSubItem = 2;
    
    ListView_InsertColumn(m_hListView, 2, &columnInfo);

    //
    // gather data
    //
    CallbackData cd = { 0 };
    cd.hListView = m_hListView;
    
    HRESULT hr = EnumLSData(ELD_MODULES, (FARPROC)ModulesCallback, (LPARAM)&cd);
    
    if (FAILED(hr))
    {
        char szBuffer[MAX_ERROR] = { 0 };
        GetSystemString(hr, szBuffer, MAX_ERROR);
        
        LVITEM itemInfo;
        itemInfo.mask = LVIF_TEXT;
        itemInfo.iItem = cd.nItem;
        itemInfo.pszText = szBuffer;
        itemInfo.iSubItem = 0;
        
        ListView_InsertItem(m_hListView, &itemInfo);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnRightClick
//
void ModulesItem::OnRightClick(int iItem)
{
    POINT cursorPos;
    GetCursorPos(&cursorPos);
    
    HMENU hMenu = CreatePopupMenu();
    
    const NamedValue<UINT> menuItems[] =
    {
        _T("&Copy path to clipboard"),      IDM_COPYMODULEPATH,
        _T("&Copy ID string to clipboard"), IDM_COPYIDSTRING,
        _T("&Unload module"),               IDM_UNLOADMODULE,
        _T("&Reload module"),               IDM_RELOADMODULE
    };
    
    for (size_t i = 0; i < COUNTOF(menuItems); ++i)
    {
        AppendMenu(hMenu, MF_STRING, menuItems[i].value, menuItems[i].name);
    }
    
    int menuItem = _TrackPopupMenu(hMenu, cursorPos);
    
    if (menuItem)
    {
        char szModulePath[MAX_PATH] = { 0 };
        ListView_GetItemText(m_hListView, iItem, 0, szModulePath, MAX_PATH);
        
        switch (menuItem)
        {
            case IDM_COPYMODULEPATH:
            {
                CopyToClipboard(m_hBox, szModulePath);
            }
            break;
            
            case IDM_COPYIDSTRING:
            {
                char szFile[MAX_PATH] = { 0 };
                char szVersion[MAX_PATH] = { 0 };
                char szThreaded[MAX_PATH] = { 0 };

                ListView_GetItemText(m_hListView, iItem, 1, szVersion, MAX_PATH);
                ListView_GetItemText(m_hListView, iItem, 2, szThreaded, MAX_PATH);

                StringCchCopy(szFile, MAX_PATH, FindFileName(szModulePath));
                
                // remove the .dll postfix
                char* pszExtension = strrchr(szFile, '.');

                if (pszExtension)
                {
                    *pszExtension = '\0';
                }

                // formatting could be better
                char szBuffer[MAX_PATH] = { 0 };
                StringCchPrintf(szBuffer, MAX_PATH, "%s--[%s]--threaded: %s",
                    szFile, szVersion, szThreaded);

                CopyToClipboard(m_hBox, szBuffer);
            }
            break;

            case IDM_UNLOADMODULE:
            {
                SendMessage(GetLitestepWnd(), LM_UNLOADMODULE,
                    (WPARAM)szModulePath, 0);
                
                // remove the module from the list
                SendMessage(m_hListView, LVM_DELETEITEM, (WPARAM) iItem, 0);
            }
            break;
            
            case IDM_RELOADMODULE:
            {
                SendMessage(GetLitestepWnd(), LM_RELOADMODULE,
                    (WPARAM)szModulePath, 0);
                
                SendMessage(m_hBox, WM_COMMAND,
                    MAKEWPARAM(IDC_COMBOBOX, CBN_SELCHANGE), 0);
            }
            break;
            
            DEFAULT_NOTREACHABLE;
        }
    }
    
    DestroyMenu(hMenu);
}
