//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  utility.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "utility.hpp"
#include <math.h>


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetClientWidth
//
// Simplified version of CreateFont
//
HFONT CreateSimpleFont(LPCTSTR faceName, int sizeInPoints, bool bold)
{
	// convert size from points to pixels
	HDC hDC = GetDC(NULL);
	
    int sizeInPixels = -MulDiv(sizeInPoints, GetDeviceCaps(hDC, LOGPIXELSY), 72);
	
    ReleaseDC(NULL, hDC);

	// fill in LOGFONT structure
    LOGFONT lf = { 0 };

	lf.lfHeight = sizeInPixels;
	lf.lfWeight = bold ? FW_BOLD : FW_NORMAL;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, faceName);

	// create it
	return CreateFontIndirect(&lf);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetClientWidth
//
// Return the width of the window's client area
//
LONG GetClientWidth(HWND hWnd)
{
    RECT r = { 0 };
	GetClientRect(hWnd, &r);

	return r.right - r.left;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// TrimLeft
//
// Trims whitespace from the beginning of a string in-place
//
void TrimLeft(char* toTrim)
{
	char* trimmed = toTrim;

	// skip past spaces
	while(*toTrim && isspace(*toTrim))
    {
        ++toTrim;
    }

	// copy the rest of the string over
	while(*toTrim)
    {
        *trimmed++ = *toTrim++;
    }

	// null-terminate it
	*trimmed = 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// FormatBytes
//
// Formats a byte count into a string suitable for display to the user
//
void FormatBytes(DWORD bytes, LPTSTR buffer, size_t cchBuffer)
{
    const TCHAR* units[] = { _T("bytes"), _T("KB"), _T("MB"),
                             _T("GB"),    _T("TB"), _T("PB") };

	double value = (double)bytes;
	unsigned int unit = 0;

	while ((value >= 1024) && unit < COUNTOF(units))
	{
		value /= 1024;
		++unit;
	}

	StringCchPrintf(buffer, cchBuffer, _T("%d %s"),
        (int)floor(value + 0.5), units[unit]);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// CopyToClipboard
//
void CopyToClipboard(HWND hWnd, LPCTSTR pszText)
{
	if(OpenClipboard(hWnd))
	{
		EmptyClipboard();
		
		HGLOBAL hGlobal =
            GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT, strlen(pszText)+1);
		
		if (hGlobal)
		{
			TCHAR* lock = (LPTSTR)GlobalLock(hGlobal);
			memcpy(lock, pszText, strlen(pszText)+1);
			
            GlobalUnlock(hGlobal);
			SetClipboardData(CF_TEXT, hGlobal);
		}

		CloseClipboard();
	}
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetSystemString
//
bool GetSystemString(DWORD dwCode, LPTSTR ptzBuffer, size_t cchBuffer)
{
    ASSERT(ptzBuffer); ASSERT(cchBuffer > 0);

    int nChars = FormatMessage(
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwCode,
        0,
        ptzBuffer,
        cchBuffer,
        NULL
        );

    if (nChars >= 2)
    {
        // clear the trailing \n
        ptzBuffer[nChars-2] = '\0';
    }

    return nChars != 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// CenterWindow
//
bool CenterWindow(HWND hWnd)
{
    ASSERT(IsWindow(hWnd));

    RECT rc = { 0 };
    
    GetWindowRect(hWnd, &rc);

    return SetWindowPos(hWnd, NULL,
        (GetSystemMetrics(SM_CXSCREEN) - (rc.right - rc.left)) / 2,
        (GetSystemMetrics(SM_CYSCREEN) - (rc.bottom - rc.top)) / 2, 
        0, 0,
        SWP_NOSIZE | SWP_NOSENDCHANGING | SWP_NOACTIVATE | SWP_NOZORDER) != FALSE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Error
//
void Error(const char* pszFormat, ...)
{
    char szMessage[MAX_PATH] = { 0 };
    va_list argList;
    
    va_start(argList, pszFormat);
    StringCchVPrintf(szMessage, MAX_PATH, pszFormat, argList);
    va_end(argList);
    
    MessageBox(NULL, szMessage, "aboutex.dll Error",
        MB_TOPMOST | MB_SETFOREGROUND | MB_ICONERROR);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetMaxWidth
//
LONG GetMaxWidth(HWND hListView)
{
    return GetClientWidth(hListView) - GetSystemMetrics(SM_CXVSCROLL);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetBasicColumn
//
// not very efficient but it doesn't have to be
//
LVCOLUMN GetBasicColumn(HWND hListView, LPTSTR pszTitle)
{
    LVCOLUMN columnInfo = { 0 };
    
    columnInfo.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    columnInfo.fmt = LVCFMT_LEFT;
    columnInfo.cx = GetMaxWidth(hListView);
    columnInfo.pszText = pszTitle;
    columnInfo.iSubItem = 0;
    
    return columnInfo;
}
